@@ -3,7 +3,7 @@ import numpy as np
 from autograd import grad, jacobian
 from collections import namedtuple
 
-from . import utils, DesignRegion
+from . import utils, DesignRegion, LDOS
 
 class OptimizationProblem(object):
     """Top-level class in the MEEP adjoint module.
@@ -119,13 +119,11 @@ class OptimizationProblem(object):
                 print("Starting forward run...")
                 self.forward_run()
                 print("Starting adjoint run...")
-                self.D_a = []
                 self.adjoint_run()
                 print("Calculating gradient...")
                 self.calculate_gradient()
             elif self.current_state == "FWD":
                 print("Starting adjoint run...")
-                self.D_a = []
                 self.adjoint_run()
                 print("Calculating gradient...")
                 self.calculate_gradient()
@@ -168,7 +166,7 @@ class OptimizationProblem(object):
             self.forward_monitors.append(m.register_monitors(self.frequencies))
 
         # register design region
-        self.design_region_monitors = utils.install_design_region_monitors(
+        self.forward_design_region_monitors = utils.install_design_region_monitors(
             self.sim, self.design_regions, self.frequencies, self.decimation_factor
         )
 
@@ -177,11 +175,16 @@ class OptimizationProblem(object):
         self.prepare_forward_run()
 
         # Forward run
-        self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
-            self.decay_by,
-            self.minimum_run_time,
-            self.maximum_run_time
-        ))
+        if any(isinstance(m, LDOS) for m in self.objective_arguments):
+            self.sim.run(mp.dft_ldos(self.frequencies), until_after_sources=mp.stop_when_dft_decayed(
+                self.decay_by,
+                self.minimum_run_time,
+                self.maximum_run_time))
+        else:
+            self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
+                self.decay_by,
+                self.minimum_run_time,
+                self.maximum_run_time))
 
         # record objective quantities from user specified monitors
         self.results_list = []
@@ -192,10 +195,7 @@ class OptimizationProblem(object):
         self.f0 = [fi(*self.results_list) for fi in self.objective_functions]
         if len(self.f0) == 1:
             self.f0 = self.f0[0]
-
-        # Store forward fields for each set of design variables in array
-        self.D_f = utils.gather_design_region_fields(self.sim,self.design_region_monitors,self.frequencies)
-
+        
         # store objective function evaluation in memory
         self.f_bank.append(self.f0)
 
@@ -225,19 +225,22 @@ class OptimizationProblem(object):
 
         # flip the k point
         if self.sim.k_point:
-            self.sim.k_point *= -1
+            self.sim.change_k_point(-1*self.sim.k_point)
 
+        self.adjoint_design_region_monitors = []
         for ar in range(len(self.objective_functions)):
             # Reset the fields
-            self.sim.reset_meep()
+            self.sim.restart_fields()
+            self.sim.clear_dft_monitors()
 
             # Update the sources
             self.sim.change_sources(self.adjoint_sources[ar])
 
-            # register design flux
-            self.design_region_monitors = utils.install_design_region_monitors(
+            # register design dft fields
+            self.adjoint_design_region_monitors.append(utils.install_design_region_monitors(
             self.sim, self.design_regions, self.frequencies, self.decimation_factor
-            )
+            ))
+            self.sim._evaluate_dft_objects()
 
             # Adjoint run
             self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
@@ -246,9 +249,6 @@ class OptimizationProblem(object):
                 self.maximum_run_time
             ))
 
-            # Store adjoint fields for each design set of design variables
-            self.D_a.append(utils.gather_design_region_fields(self.sim,self.design_region_monitors,self.frequencies))
-
         # reset the m number
         if utils._check_if_cylindrical(self.sim):
             self.sim.m = -self.sim.m
@@ -264,8 +264,8 @@ class OptimizationProblem(object):
         self.gradient = [[
             dr.get_gradient(
                 self.sim,
-                self.D_a[ar][dri],
-                self.D_f[dri],
+                self.adjoint_design_region_monitors[ar][dri],
+                self.forward_design_region_monitors[dri],
                 self.frequencies,
                 self.finite_difference_step
             ) for dri, dr in enumerate(self.design_regions)
@@ -312,11 +312,22 @@ class OptimizationProblem(object):
         '''
         if filter is None:
             filter = lambda x: x
-        if num_gradients > self.num_design_params[design_variables_idx]:
+        if num_gradients < self.num_design_params[design_variables_idx]:
+            # randomly choose indices to loop estimate
+            fd_gradient_idx = np.random.choice(
+                self.num_design_params[design_variables_idx],
+                num_gradients,
+                replace=False,
+            )
+        elif num_gradients == self.num_design_params[design_variables_idx]:
+            fd_gradient_idx = range(self.num_design_params[design_variables_idx])
+        else:
             raise ValueError(
                 "The requested number of gradients must be less than or equal to the total number of design parameters."
             )
 
+        assert db < 0.2, "The step size of finite difference is too large."   
+
         # cleanup simulation object
         self.sim.reset_meep()
         self.sim.change_sources(self.forward_sources)
@@ -324,13 +335,6 @@ class OptimizationProblem(object):
         # preallocate result vector
         fd_gradient = []
 
-        # randomly choose indices to loop estimate
-        fd_gradient_idx = np.random.choice(
-            self.num_design_params[design_variables_idx],
-            num_gradients,
-            replace=False,
-        )
-
         for k in fd_gradient_idx:
 
             b0 = np.ones((self.num_design_params[design_variables_idx], ))
@@ -342,7 +346,10 @@ class OptimizationProblem(object):
             self.sim.reset_meep()
 
             # assign new design vector
-            b0[k] -= db
+            in_interior = True  # b0[k] is not too close to the boundaries 0 and 1
+            if b0[k] < db or b0[k]+db > 1: in_interior = False  # b0[k] is too close to 0 or 1
+
+            if b0[k] >= db: b0[k] -= db
             self.design_regions[design_variables_idx].update_design_parameters(
                 b0)
 
@@ -352,11 +359,13 @@ class OptimizationProblem(object):
                 self.forward_monitors.append(
                     m.register_monitors(self.frequencies))
 
-            self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
-                self.decay_by,
-                self.minimum_run_time,
-                self.maximum_run_time
-            ))
+            if any(isinstance(m, LDOS) for m in self.objective_arguments):
+                self.sim.run(mp.dft_ldos(self.frequencies), until_after_sources=mp.stop_when_energy_decayed(dt=1, decay_by=1e-11))
+            else:
+                self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
+                    self.decay_by,
+                    self.minimum_run_time,
+                    self.maximum_run_time))
 
             # record final objective function value
             results_list = []
@@ -370,7 +379,9 @@ class OptimizationProblem(object):
             self.sim.reset_meep()
 
             # assign new design vector
-            b0[k] += 2 * db  # central difference rule...
+            if in_interior: b0[k] += 2 * db  # central difference rule...
+            else: b0[k] += db  # forward or backward  difference...
+
             self.design_regions[design_variables_idx].update_design_parameters(
                 b0)
 
@@ -381,11 +392,13 @@ class OptimizationProblem(object):
                     m.register_monitors(self.frequencies))
 
             # add monitor used to track dft convergence
-            self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
-                self.decay_by,
-                self.minimum_run_time,
-                self.maximum_run_time
-            ))
+            if any(isinstance(m, LDOS) for m in self.objective_arguments):
+                self.sim.run(mp.dft_ldos(self.frequencies), until_after_sources=mp.stop_when_energy_decayed(dt=1, decay_by=1e-11))
+            else:
+                self.sim.run(until_after_sources=mp.stop_when_dft_decayed(
+                    self.decay_by,
+                    self.minimum_run_time,
+                    self.maximum_run_time))
 
             # record final objective function value
             results_list = []
@@ -397,7 +410,7 @@ class OptimizationProblem(object):
             # estimate derivative
             # -------------------------------------------- #
             fd_gradient.append([
-                np.squeeze((fp[fi] - fm[fi]) / (2 * db))
+                np.squeeze((fp[fi] - fm[fi]) / db / (2 if in_interior else 1))
                 for fi in range(len(self.objective_functions))
             ])
 
