@@ -53,7 +53,7 @@ enum component {
   Permeability,
   NO_COMPONENT
 };
-#define Centered Dielectric // better name for centered "dielectric" grid
+const component Centered = Dielectric; // better name for centered "dielectric" grid
 enum derived_component {
   Sx = 100,
   Sy,
@@ -349,16 +349,16 @@ _Pragma("omp parallel for collapse(2)")
    (loop_s3 != 0 && (loop_i3 == 0 || loop_i3 == loop_n3 - 1)))
 
 #define IVEC_LOOP_ILOC(gv, iloc)                                                                   \
-  ivec iloc((gv).dim);                                                                             \
-  iloc.set_direction(direction(loop_d1), loop_is1 + 2 * loop_i1);                                  \
-  iloc.set_direction(direction(loop_d2), loop_is2 + 2 * loop_i2);                                  \
-  iloc.set_direction(direction(loop_d3), loop_is3 + 2 * loop_i3)
+  meep::ivec iloc((gv).dim);                                                                             \
+  iloc.set_direction(meep::direction(loop_d1), loop_is1 + 2 * loop_i1);                                  \
+  iloc.set_direction(meep::direction(loop_d2), loop_is2 + 2 * loop_i2);                                  \
+  iloc.set_direction(meep::direction(loop_d3), loop_is3 + 2 * loop_i3)
 
 #define IVEC_LOOP_LOC(gv, loc)                                                                     \
-  vec loc((gv).dim);                                                                               \
-  loc.set_direction(direction(loop_d1), (0.5 * loop_is1 + loop_i1) * (gv).inva);                   \
-  loc.set_direction(direction(loop_d2), (0.5 * loop_is2 + loop_i2) * (gv).inva);                   \
-  loc.set_direction(direction(loop_d3), (0.5 * loop_is3 + loop_i3) * (gv).inva)
+  meep::vec loc((gv).dim);                                                                               \
+  loc.set_direction(meep::direction(loop_d1), (0.5 * loop_is1 + loop_i1) * (gv).inva);                   \
+  loc.set_direction(meep::direction(loop_d2), (0.5 * loop_is2 + loop_i2) * (gv).inva);                   \
+  loc.set_direction(meep::direction(loop_d3), (0.5 * loop_is3 + loop_i3) * (gv).inva)
 
 // integration weight for using LOOP_OVER_IVECS with field::integrate
 #define IVEC_LOOP_WEIGHT1x(s0, s1, e0, e1, i, n, dir)                                              \
@@ -412,6 +412,8 @@ inline bool is_electric(component c) { return c < Hx; }
 inline bool is_magnetic(component c) { return c >= Hx && c < Dx; }
 inline bool is_D(component c) { return c >= Dx && c < Bx; }
 inline bool is_B(component c) { return c >= Bx && c < Dielectric; }
+inline bool is_E_or_D(component c) {return is_electric(c) || is_D(c); }
+inline bool is_H_or_B(component c) {return is_magnetic(c) || is_B(c); }
 inline bool is_derived(int c) { return c >= Sx; }
 inline bool is_poynting(derived_component c) { return c < EnergyDensity; }
 inline bool is_energydensity(derived_component c) { return c >= EnergyDensity; }
@@ -884,10 +886,10 @@ public:
   // case not thread-safe)
   const char *str(char *buffer = 0, size_t buflen = 0);
 
-  ivec round_up_to_even(void) const {
+  ivec round_down_to_even(void) const {
     ivec result(dim);
     LOOP_OVER_DIRECTIONS(dim, d)
-    result.t[d] = t[d] + (t[d] >= 0 ? t[d] : -t[d]) % 2;
+    result.t[d] = t[d] - (t[d] >= 0 ? t[d] : -t[d]) % 2;
     return result;
   }
 
@@ -1006,8 +1008,8 @@ class grid_volume {
 public:
   grid_volume(){};
 
-  grid_volume subvolume(ivec is, ivec ie);
-  void init_subvolume(ivec is, ivec ie);
+  grid_volume subvolume(ivec is, ivec ie, component c);
+  void init_subvolume(ivec is, ivec ie, component c);
 
   ndim dim;
   double a, inva /* = 1/a */;
@@ -1095,6 +1097,7 @@ public:
   }
 
   ivec little_owned_corner(component c) const;
+  ivec big_owned_corner(component c) const { return big_corner() - iyee_shift(c); }
   bool owns(const ivec &) const;
   volume surroundings() const;
   volume interior() const;
@@ -1212,12 +1215,12 @@ public:
   void operator=(const symmetry &);
   bool operator==(const symmetry &) const;
   bool operator!=(const symmetry &S) const { return !(*this == S); };
+  ivec i_symmetry_point;
 
 private:
   signed_direction S[5];
   std::complex<double> ph;
   vec symmetry_point;
-  ivec i_symmetry_point;
   int g; // g is the multiplicity of the symmetry.
   symmetry *next;
   friend symmetry r_to_minus_r_symmetry(double m);
