@@ -292,6 +292,18 @@ static int nextpow2357(int n) {
   }
 }
 
+void special_kz_phasefix(eigenmode_data *edata, bool phase_flip) {
+  size_t n = edata->n[0] * edata->n[1] * edata->n[2];
+  complex<mpb_real> *E = (complex<mpb_real> *)edata->fft_data_E;
+  complex<mpb_real> *H = (complex<mpb_real> *)edata->fft_data_H;
+  complex<mpb_real> im(0, phase_flip ? -1 : 1);
+  for (size_t i = 0; i < n; ++i) {
+    E[3*i + 2] *= im; // Ez
+    H[3*i + 0] *= im; // Hx
+    H[3*i + 1] *= im; // Hy
+  }
+}
+
 /****************************************************************/
 /* call MPB to get the band_numth eigenmode at freq frequency.  */
 /*                                                              */
@@ -322,9 +334,16 @@ void *fields::get_eigenmode(double frequency, direction d, const volume where, c
   // if the mode region extends over the full computational grid and we are bloch-periodic
   // in any direction, set the corresponding component of the eigenmode initial-guess
   // k-vector to be the (real part of the) bloch vector in that direction.
+  grid_volume eig_gv;
+  if (eig_vol.dim == D1)
+    eig_gv = vol1d(eig_vol.in_direction(Z), a);
+  else if (eig_vol.dim == D2)
+    eig_gv = vol2d(eig_vol.in_direction(X), eig_vol.in_direction(Y), a);
+  else
+    eig_gv = vol3d(eig_vol.in_direction(X), eig_vol.in_direction(Y), eig_vol.in_direction(Z), a);
   vec kpoint(_kpoint);
   LOOP_OVER_DIRECTIONS(v.dim, dd) {
-    if (dd != d && float(eig_vol.in_direction(dd)) == float(v.in_direction(dd)))
+    if (dd != d && eig_gv.num_direction(dd) == user_volume.num_direction(dd))
       if (boundaries[High][dd] == Periodic && boundaries[Low][dd] == Periodic)
         kpoint.set_direction(dd, real(k[dd]));
   }
@@ -332,12 +351,10 @@ void *fields::get_eigenmode(double frequency, direction d, const volume where, c
   bool empty_dim[3] = {false, false, false};
 
   // special case: 2d cell in x and y with non-zero kz
-  if ((eig_vol.dim == D3) && (float(v.in_direction(Z)) == float(1 / a)) &&
-      (boundaries[High][Z] == Periodic && boundaries[Low][Z] == Periodic) && (kpoint.z() == 0) &&
-      (real(k[Z]) != 0)) {
-    kpoint.set_direction(Z, real(k[Z]));
+  if ((v.dim == D3) && (float(v.in_direction(Z)) == float(1 / a)) &&
+      (boundaries[High][Z] == Periodic && boundaries[Low][Z] == Periodic) &&
+      (real(k[Z]) != 0))
     empty_dim[2] = true;
-  }
 
   if (resolution <= 0.0) resolution = 2 * gv.a; // default to twice resolution
   int n[3], local_N, N_start, alloc_N, mesh_size[3] = {1, 1, 1};
@@ -604,16 +621,29 @@ void *fields::get_eigenmode(double frequency, direction d, const volume where, c
     int m = 0;
     LOOP_OVER_DIRECTIONS(v.dim, dd) {
       m = dp->get_g()[dd - X];
-      ktmp = kpoint.in_direction(dd);
-      if (eig_vol.in_direction(dd) != 0)
-        ktmp += m/eig_vol.in_direction(dd);
-      k2sum += ktmp*ktmp;
+      if (eig_vol.in_direction(dd) != 0) {
+        ktmp = kpoint.in_direction(dd) + m/eig_vol.in_direction(dd);
+        k2sum += ktmp*ktmp;
+      }
+    }
+    if (((v.dim == D3) && (float(v.in_direction(Z)) == float(1 / a)) &&
+        (boundaries[High][Z] == Periodic && boundaries[Low][Z] == Periodic) && (real(k[Z]) != 0)) ||
+        ((v.dim == D2) && (real(k[Z]) != 0))) {
+      k2sum += k[Z]*k[Z];
     }
 
     // compute kperp (if it is non evanescent) OR
     // frequency from kperp^2 and sum of (kparallel+G)^2
     {
       direction dd = eig_vol.normal_direction();
+      if (eig_vol.dim == D3 && empty_dim[2]) {
+        volume eig_vol_2d(D2);
+        eig_vol_2d.set_direction_min(X, eig_vol.in_direction_min(X));
+        eig_vol_2d.set_direction_min(Y, eig_vol.in_direction_min(Y));
+        eig_vol_2d.set_direction_max(X, eig_vol.in_direction_max(X));
+        eig_vol_2d.set_direction_max(Y, eig_vol.in_direction_max(Y));
+        dd = eig_vol_2d.normal_direction();
+      }
       if (match_frequency) {
         vec cen = eig_vol.center();
         double nn = sqrt(real(get_eps(cen, frequency)) * real(get_mu(cen, frequency)));
@@ -809,6 +839,8 @@ void fields::add_eigenmode_source(component c0, const src_time &src, direction d
                                       NULL, NULL, dp);
   finished_working();
 
+  if (is_real && beta != 0) special_kz_phasefix(global_eigenmode_data, true /* phase_flip */);
+
   if (global_eigenmode_data == NULL) meep::abort("MPB could not find the eigenmode");
 
   /* add_volume_source amp_fun coordinates are relative to where.center();
@@ -923,6 +955,8 @@ void fields::get_eigenmode_coefficients(dft_flux flux, const volume &eig_vol, in
         continue;
       }
 
+      if (is_real && beta != 0) special_kz_phasefix((eigenmode_data *) mode_data, false /* phase_flip */);
+
       double vg = get_group_velocity(mode_data);
       vec kfound = get_k(mode_data);
       if (vgrp) vgrp[nb * num_freqs + nf] = vg;
